/**
 *    Copyright (C) 2012, 2013 Engineering Ingeneria Informatica s.p.a.
 *
 *    Authors:
 *     Paolo Roccetti <paolo.roccetti@eng.it>
 *     Silvio Sorace <silvio.sorace@eng.it>
 *
 *    This file is part of SEMIRAMIS Policy Management library (PM).
 *
 *    PM is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU Lesser General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    PM is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *    GNU Lesser General Public License for more details.
 *
 *    You should have received a copy of the GNU Lesser General Public License
 *    along with PM. If not, see <http://www.gnu.org/licenses/>.
 * 
 */
package eu.semiramis.policyManager.pdp.an;

import an.xacml.engine.PDPStatus;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.InputStream;

import org.opensaml.xacml.ctx.RequestType;
import org.opensaml.xacml.ctx.ResponseType;
import org.opensaml.xml.io.MarshallingException;
import org.opensaml.xml.io.UnmarshallingException;
import org.opensaml.xml.parse.XMLParserException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import an.config.ConfigElement;
import an.control.OperationFailedException;
import an.xacml.adapter.file.XACMLParser;
import an.xacml.context.Request;
import an.xacml.context.Response;
import an.xacml.engine.PDP;
import an.xacml.engine.PDPInitializeException;
import an.xacml.engine.PDPNotReadyException;
import an.xml.XMLAttribute;
import an.xml.XMLElement;
import an.xml.XMLGeneralException;
import eu.semiramis.policyManager.pdp.ConfigurationException;
import eu.semiramis.policyManager.pdp.EvaluationException;
import eu.semiramis.policyManager.pdp.impl.AbstractPDP;
import eu.semiramis.policyManager.util.opensaml.XACMLResponseUtils;
import eu.semiramis.policyManager.util.opensaml.XACMLUtils;

/**
 * This concrete implementation of the PDP uses an Enterprise Java XACML engine
 * to evaluate access control requests against the current XACML policies
 * 
 */
public class AnPDP extends AbstractPDP implements eu.semiramis.policyManager.pdp.PDP {

	private PDP pdp;

	final Logger logger = LoggerFactory.getLogger(AnPDP.class);

	private long lastKnownChangeTime;

	private String policyDir;

	/**
	 * Creates a new AnPDP instance.
	 * 
	 * @param policyDir
	 *            the directory containing XACML policies to enforce
	 */
	public AnPDP(String pdpName, String ContextHandlerId, String policyDir) throws ConfigurationException {
		super(pdpName);
		
		this.policyDir = policyDir;
		
		// load default configuration file and get the pdp configuration element
		ConfigElement pdpConfigElement = (ConfigElement) AnPDPFactory
				.loadDefaultConfiguration().getConfigurationElement()
				.getXMLElementsByName(PDP.ELEM_PDP)[0];

		// find attributes of the FileDataStore element
		XMLElement fileDataStoreElem;
		try {
			fileDataStoreElem = pdpConfigElement
					.getSingleXMLElementByName("FileDataStore");
		} catch (XMLGeneralException e) {
			throw new ConfigurationException(
					"Cannot find FileDataStore element inside the default EnterpriseJavaXACML PDP config",
					e);
		}

		XMLAttribute[] fileDataStoreAttributes = fileDataStoreElem
				.getAttributes();

		// set policyDir in the "path" attribute of the FileDataStore
		// element
		for (int i = 0; i < fileDataStoreAttributes.length; i++) {
			if (fileDataStoreAttributes[i].getName().equals("path")) {
				fileDataStoreAttributes[i] = new XMLAttribute("path", policyDir);
			}
		}

		
		XMLAttribute[] attributes = pdpConfigElement.getAttributes();
		
		//set pdp name in domainName attribute of PDP element
		for (int i = 0; i < attributes.length; i++) {
			if (attributes[i].getName().equals("domainName")){
				attributes[i] = new XMLAttribute("domainName", ContextHandlerId + "-" + pdpName);
			}
		}
			
		// EnterpriseJavaXACML PDP instantiation
		try {
			pdp = an.xacml.engine.PDP.getInstance(pdpConfigElement);
		} catch (PDPInitializeException e) {
			throw new ConfigurationException(
					"Cannot initialize the EnterpriseJavaXACML PDP", e);
		}
		
		//calculate last change time for policy dir
		lastKnownChangeTime = (new File(policyDir)).lastModified();
		
		try {
			pdp.start();
		} catch (OperationFailedException e) {
			throw new ConfigurationException(
			"Cannot start the EnterpriseJavaXACML PDP", e);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see eu.semiramis.policyManager.pdp.PDP#evaluate(org.opensaml.xacml.ctx.
	 * RequestType)
	 */
	@Override
	public ResponseType evaluate(RequestType request)
			throws EvaluationException {

		logger.info("translating openSAML request {} to EnterpriseJavaXACML",
				request);

		// translate request from openSAML to EnterpriseJavaXACML
		Request anRequest;
		try {
			anRequest = translate(request);
		} catch (Exception e) {
			throw new EvaluationException(
					"Cannot translate openSAML request to the EnterpriseJavaXACML format: "
							+ e.getMessage(), e);
		}

		logger.info(
				"openSAML request {} translated to EnterpriseJavaXACML request {}:\n{}",
				new Object[] { request, anRequest, toPrint(anRequest) });

		//calculate last change time for policy dir
		long lastChangeTime = (new File(policyDir)).lastModified();
		
		//Reload policies if needed and update lastKnownChangeTime variable
		if (lastChangeTime > lastKnownChangeTime) {
			try {
				logger.info("Reloading policies for PDP " + getName());				
				pdp.reloadPolicies();
				logger.info("Reloaded policies for PDP " + getName());
			} catch (Exception e) {
				throw new EvaluationException(
						"The " + getName() + " PDP failed to reload policies",
						e);
			}
			
			//update lastKnownChangeTime variable
			lastKnownChangeTime = lastChangeTime;
		}
		
		// evaluate request
		Response anResponse;
		try {
			anResponse = (Response) pdp.handleRequest(anRequest);
		} catch (Exception e) {
			throw new EvaluationException(
					"The " + getName() + " PDP failed to evaluate request",
					e);
		}

		logger.info(
				"evaluation of request {} using EnterpriseJavaXACML returned {}:\n{}",
				new Object[] { anRequest, anResponse, toPrint(anResponse) });

		logger.info("translating EnterpriseJavaXACML response {} to openSAML",
				anResponse);

		// translate response from EnterpriseJavaXACML to openSAML
		ResponseType response;
		try {
			response = translate(anResponse);
		} catch (Exception e) {
			throw new EvaluationException(
					"Cannot translate EnterpriseJavaXACML response to openSAML format: "
							+ e.getMessage(), e);
		}

		logger.info(
				"EnterpriseJavaXACML response {} translated to openSAML response {}",
				anResponse, response);

		return response;
	}

	/**
	 * translate an openSAML request to an EnterpriseJavaXACML request
	 * 
	 * @param request
	 *            the openSAML request to translate
	 * @return the translated EnterpriseJavaXACML request
	 * @throws MarshallingException
	 *             if an exception occurs during openSAML marshalling
	 * @throws AnMarshallingException
	 *             if an exception occurs during EnterpriseJavaXACML
	 *             unmarshalling
	 */
	private Request translate(RequestType request) throws MarshallingException,
			AnMarshallingException {

		String strRequest = XACMLUtils.printXMLObject(request);

		InputStream requestStream = new ByteArrayInputStream(
				strRequest.getBytes());

		try {
			return XACMLParser.parseRequest(requestStream);
		} catch (Exception e) {
			throw new AnMarshallingException(
					"cannot unmarshall EnterpriseJavaXACML request", e);
		}
	}

	/**
	 * translate an EnterpriseJavaXACML response to an openSAML response
	 * 
	 * @param anResponse
	 *            the EnterpriseJavaXACML response to translate
	 * @return the translated openSAML response
	 * @throws UnmarshallingException
	 *             if an exception occurs during openSAML unmarshalling
	 * @throws XMLParserException
	 *             if an exception occurs during openSAML parsing
	 * @throws AnMarshallingException
	 *             if an exception occurs during EnterpriseJavaXACML marshalling
	 */
	private ResponseType translate(Response anResponse)
			throws XMLParserException, UnmarshallingException,
			AnMarshallingException {

		// marshall the response to a string
		String strResponse;
		try {
			strResponse = marshall(anResponse);
		} catch (Exception e) {
			throw new AnMarshallingException(
					"cannot marshall EnterpriseJavaXACML response", e);
		}

		// create the openSAML response
		ResponseType openSAMLResponse = XACMLResponseUtils
				.deserialize(new ByteArrayInputStream(strResponse.getBytes()));

		// return the response
		return openSAMLResponse;

	}

	/**
	 * prepare an EnterpriseJavaXACML request to be printed
	 * 
	 * @param anRequest
	 *            the EnterpriseJavaXACML request to print
	 * @return the marshalled EnterpriseJavaXACML request, or a string reporting
	 *         the problem occurred
	 */
	private String toPrint(Request anRequest) {
		try {
			return marshall(anRequest);
		} catch (Exception e) {
			return "cannot prepare EnterpriseJavaXACML request " + anRequest
					+ " to be printed: " + e.getMessage();
		}
	}

	/**
	 * prepare an EnterpriseJavaXACML response to be printed
	 * 
	 * @param anResponse
	 *            the EnterpriseJavaXACML response to print
	 * @return the marshalled EnterpriseJavaXACML response, or a string
	 *         reporting the problem occurred
	 */
	private String toPrint(Response anResponse) {
		try {
			return marshall(anResponse);
		} catch (Exception e) {
			return "cannot prepare EnterpriseJavaXACML response " + anResponse
					+ " to be printed: " + e.getMessage();
		}
	}

	/**
	 * marshall an EnterpriseJavaXACML request to a string
	 * 
	 * @param anRequest
	 *            the EnterpriseJavaXACML request to marshall
	 * @return the marshalled EnterpriseJavaXACML request
	 * @throws Exception
	 *             if an exception occurs during marshalling
	 */
	private String marshall(Request anRequest) throws Exception {
		ByteArrayOutputStream tempOut = new ByteArrayOutputStream();
		XACMLParser.dumpRequest(anRequest, tempOut);
		return tempOut.toString();
	}

	/**
	 * marshall an EnterpriseJavaXACML response to a string
	 * 
	 * @param anResponse
	 *            the EnterpriseJavaXACML response to marshall
	 * @return the marshalled EnterpriseJavaXACML response
	 * @throws Exception
	 *             if an exception occurs during marshalling
	 */
	private String marshall(Response anResponse) throws Exception {
		ByteArrayOutputStream tempOut = new ByteArrayOutputStream();
		XACMLParser.dumpResponse(anResponse, tempOut);
		return tempOut.toString();
	}

	
	/* (non-Javadoc)
	 * @see eu.semiramis.policyManager.pdp.PDP#destroy()
	 */
	@Override
	public void destroy() {
		pdp.shutdown();
		
	}
}
