/**
 *    Copyright (C) 2012, 2013 Engineering Ingeneria Informatica s.p.a.
 *
 *    Authors:
 *     Paolo Roccetti <paolo.roccetti@eng.it>
 *     Silvio Sorace <silvio.sorace@eng.it>
 *
 *    This file is part of SEMIRAMIS Policy Management library (PM).
 *
 *    PM is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU Lesser General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    PM is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *    GNU Lesser General Public License for more details.
 *
 *    You should have received a copy of the GNU Lesser General Public License
 *    along with PM. If not, see <http://www.gnu.org/licenses/>.
 * 
 */
package eu.semiramis.policyManager.pdp.an;

import java.util.Properties;

import org.opensaml.xacml.ctx.RequestType;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import an.config.ConfigElement;
import an.config.Configuration;
import an.log.LogFactory;
import eu.semiramis.policyManager.pdp.ConfigurationException;
import eu.semiramis.policyManager.pdp.ContextHandler;
import eu.semiramis.policyManager.pdp.PDP;
import eu.semiramis.policyManager.pdp.PDPFactory;

/**
 * This concrete implementation of the {@link PDPFactory} creates a
 * {@link AnPDP} that uses an Enterprise Java XACML engine to evaluate access
 * control requests against the current XACML policies.
 * <p>
 * The "pdp.factory.an.policy.dir" property can be used in the configuration
 * file to set the directory where XACML policies must be loaded from.
 * 
 */
public class AnPDPFactory implements PDPFactory {

	public static final String AN_POLICY_DIR_PROPERTY = ".factory.an.policy.dir";

	final Logger logger = LoggerFactory.getLogger(AnPDPFactory.class);

	private static boolean anInitialized;

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * eu.semiramis.policyManager.pdp.PDPFactory#getPDP(java.util.Properties,
	 * org.opensaml.xacml.ctx.RequestType)
	 */
	@Override
	public PDP getPDP(String pdpName, ContextHandler handler, Properties configurationProperties, RequestType request)
			throws ConfigurationException {

		// Initialize the An library
		if (!anInitialized)
			initializeAn();

		logger.info("creating new AnPDP for request {} using properties {}",
				request, configurationProperties);

		// get directory form properties
		String policyDir = configurationProperties
				.getProperty(pdpName + AnPDPFactory.AN_POLICY_DIR_PROPERTY);

		logger.info("policyDir {}",	policyDir);
		
		// return a new AnPDP
		return new AnPDP(pdpName, handler.getContextHandlerId(), policyDir);

	}

	/**
	 * Initialize the Enterprise Java XACML library
	 * 
	 * @throws ConfigurationException
	 *             if an exception occurs during initialization
	 */
	private static void initializeAn() throws ConfigurationException {

		System.setProperty("an.config.Configuration.Schemas",
				"configuration.xsd");
		System.setProperty("an.xacml.context.DefaultSchema",
				"xacml-2.0-context.xsd");
		System.setProperty("an.xacml.policy.DefaultSchema",
				"xacml-2.0-policy.xsd");
		System.setProperty("an.xml.DocumentBuilderFactory.forceSUN", "");

		// load default configuration file
		Configuration config = loadDefaultConfiguration();
		try {

			ConfigElement rootConfig = config.getConfigurationElement();

			// Enterprise Java XACML logging configuration
			ConfigElement logConfig = (ConfigElement) rootConfig
					.getSingleXMLElementByType(an.log.Logger.ELEMTYPE_LOG);
			LogFactory.initialize(logConfig);
		} catch (Exception e) {
			throw new ConfigurationException(
					"Cannot initialize the EnterpriseJavaXACML library", e);
		}

		anInitialized = true;
	}

	/**
	 * This method load the default configuration for the EnterpriseJavaXACML
	 * pdp. The default configuration is stored in the classpath into the
	 * <code>eu/semiramis/policyManager/pdp/an/pdpConfig.xml</code> file
	 * 
	 * @return the default configuration for the EnterpriseJavaXACML pdp
	 * */
	static Configuration loadDefaultConfiguration() {
		try {
			return new Configuration(Thread
					.currentThread()
					.getContextClassLoader()
					.getResourceAsStream(
							"eu/semiramis/policyManager/pdp/an/pdpConfig.xml"));
		} catch (an.config.ConfigurationException e) {
			// This will not happen as the default configuration is inside the
			// policy management library
			throw new RuntimeException(
					"Cannot find the EnterpriseJavaXACML pdpConfig file! ");
		}
	}

}
