/**
 *    Copyright (C) 2012, 2013 Engineering Ingeneria Informatica s.p.a.
 *
 *    Authors:
 *     Paolo Roccetti <paolo.roccetti@eng.it>
 *     Silvio Sorace <silvio.sorace@eng.it>
 *
 *    This file is part of SEMIRAMIS Policy Management library (PM).
 *
 *    PM is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU Lesser General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    PM is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *    GNU Lesser General Public License for more details.
 *
 *    You should have received a copy of the GNU Lesser General Public License
 *    along with PM. If not, see <http://www.gnu.org/licenses/>.
 * 
 */
package eu.semiramis.policyManager.pdp.impl;

import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Properties;
import java.util.Set;
import java.util.StringTokenizer;

import org.opensaml.xacml.ctx.RequestType;
import org.opensaml.xacml.ctx.ResponseType;
import org.opensaml.xml.io.MarshallingException;
import org.slf4j.Logger;

import eu.semiramis.policyManager.pdp.ConfigurationException;
import eu.semiramis.policyManager.pdp.ContextHandler;
import eu.semiramis.policyManager.pdp.EvaluationException;
import eu.semiramis.policyManager.pdp.PDP;
import eu.semiramis.policyManager.pdp.PDPFactory;
import eu.semiramis.policyManager.util.PMUtils;
import eu.semiramis.policyManager.util.opensaml.XACMLUtils;

/**
 * Base implementation of the {@link ContextHandler} interface
 * 
 */
public abstract class AbstractContextHandler implements ContextHandler {

	protected Properties properties = null;

	//This ID allows to distinguish PDPs among different context handlers (to avoid a problem with duplication of AnPDP)
	private static Integer contextHandlerIdCounter = 0;
	
	//store the current Context Handler identifier
	private int contextHandlerId;
	
	private Map<String, PDP> pdpMap;

	private Properties lastConfiguredProperties = null;
	
	/**
	 * 
	 */
	public AbstractContextHandler() {
		
		//Increase the counter and store the new contextHandler identifier for this Context Handler instance
		synchronized (contextHandlerIdCounter) {
			contextHandlerIdCounter = contextHandlerIdCounter + 1;
			contextHandlerId = contextHandlerIdCounter;
		}
		
	}
	
	abstract Logger getLogger();
	
	/* (non-Javadoc)
	 * @see eu.semiramis.policyManager.pdp.ContextHandler#getContextHandlerId()
	 */
	@Override
	public String getContextHandlerId() {
		return "CH:" + contextHandlerId;
	}

	abstract ResponseType handle(RequestType request) throws EvaluationException;

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * eu.semiramis.policyManager.pdp.ContextHandler#setConfigurationProperties
	 * (java.util.Properties)
	 */
	@Override
	public void setConfigurationProperties(Properties properties) throws ConfigurationException {

		getLogger().info("setting in {} the new configuration properties\n{}", this,
				properties);

		this.properties = properties;

	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * eu.semiramis.policyManager.pdp.ContextHandler#handleRequest(org.opensaml
	 * .xacml.ctx.RequestType)
	 */
	public synchronized ResponseType handleRequest(RequestType request)
			throws EvaluationException {
		
		try {
			getLogger().info("start handling request {}: \n{}", request,
					XACMLUtils.printXMLObject(request));
		} catch (MarshallingException e1) {
			throw new EvaluationException("Cannot marshall request " + request, e1);
		}
		
		//if properties is null then throw nullpointerexception
		if (properties == null) throw new NullPointerException("Properties for Context Handler " + getContextHandlerId() + " are missing ");
		
		//if properties have been changed, update the Context Handler configuration
		if (!propertiesChanged()) {
			
			getLogger().info("Properties for Context Handler " + getContextHandlerId() + " have been changed. Reconfiguring...");
			
			try {
				configure();
			} catch (ConfigurationException e) {
				throw new EvaluationException("Cannot reconfigure the Context Handler to handle request " + request, e);
			}
			
			getLogger().info("Context Handler " + getContextHandlerId() + " has been reconfigured with new properties");
		}
		
		//handle request
		ResponseType response = handle(request);	
		
		try {
			getLogger().info(
					"evaluation of request {} returned the response {}: \n{}",
					new Object[] { request, response,
							XACMLUtils.printXMLObject(response) });
		} catch (MarshallingException e3) {
			throw new EvaluationException("Cannot marshall response " + response, e3);
		}

		return response;
	}

	ResponseType handleByPdp(String pdpName, RequestType request) throws EvaluationException {
		
		//get the pdp from map
		PDP pdp = pdpMap.get(pdpName);
		
		if (pdp == null) {
			throw new EvaluationException("Cannot find pdp " + pdpName + " for context handler " + getContextHandlerId());
		}
		
		getLogger().info("Context Handler " + getContextHandlerId() + " is using PDP {} to evaluate request {}", pdpName, request);

		ResponseType response = pdp.evaluate(request);
		
		return response;
	}

	/*
	 * This method configures the context handler and the PDPs 
	 */
	private void configure() throws ConfigurationException {
		
		//read the new property set and loads it in the lastConfiguredProperties.
		lastConfiguredProperties = new Properties();
		for (String key : properties.stringPropertyNames()) {
			lastConfiguredProperties.setProperty(key, properties.getProperty(key));
		}
		
		//destroy previous PDPs, if any
		if (pdpMap != null) {
			Iterator<PDP> iterator = pdpMap.values().iterator();
			while (iterator.hasNext()) {
				PDP pdp = (PDP) iterator.next();
				pdp.destroy();
				iterator.remove();
			}
		}
		
		//initialize the new PDP set
		pdpMap = new HashMap<String, PDP>();
		
		//get pdp names
		String pdpNamesProperty = lastConfiguredProperties.getProperty("pdp.names");
		StringTokenizer pdpNames = new StringTokenizer(pdpNamesProperty, ",");
		
		// iterate to create pdps
		while (pdpNames.hasMoreElements()) {
			String pdpName=pdpNames.nextToken();
			
			// get the PDPFactory
			PDPFactory pdpFactory;
			try {
				pdpFactory = PMUtils.create(PDPFactory.class, lastConfiguredProperties
						.getProperty(pdpName + PDPFactory.PDP_FACTORY_CLASS_PROPERTY));
			} catch (Exception e2) {
				throw new ConfigurationException(
						"Cannot create the PDP factory to get PDP " + pdpName,
						e2);
			}

			getLogger().info("using PDPFactory {} to get PDP {}",
					pdpFactory, pdpName);

			// get the PDP from the PDPFactory
			PDP pdp = pdpFactory.getPDP(pdpName, this, this.lastConfiguredProperties, null);
			
			//store the PDP in the Context Handler Map
			pdpMap.put(pdpName, pdp);
			
		}
		
	}
	
	
	/**
	 * @return
	 */
	private boolean propertiesChanged() {
		
		//check if lastConfiguredProperties is null
		if (lastConfiguredProperties == null) return false;
		
		//check property names for differences
		Set<String> newNames = properties.stringPropertyNames();
		Set<String> oldNames = lastConfiguredProperties.stringPropertyNames();
		if (newNames.size() != oldNames.size() || !newNames.containsAll(oldNames)) return false;
		
		//check for property values
		for (String oldName : oldNames) {
			if (!lastConfiguredProperties.getProperty(oldName).equals(properties.getProperty(oldName))) return false;
		}
		
		return true;
	}
	
}
