/**
 *    Copyright (C) 2012, 2013 Engineering Ingeneria Informatica s.p.a.
 *
 *    Authors:
 *     Paolo Roccetti <paolo.roccetti@eng.it>
 *     Silvio Sorace <silvio.sorace@eng.it>
 *
 *    This file is part of SEMIRAMIS Policy Management library (PM).
 *
 *    PM is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU Lesser General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    PM is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *    GNU Lesser General Public License for more details.
 *
 *    You should have received a copy of the GNU Lesser General Public License
 *    along with PM. If not, see <http://www.gnu.org/licenses/>.
 * 
 */
package eu.semiramis.policyManager.pdp.impl;

import java.io.IOException;
import java.net.URISyntaxException;
import java.util.Properties;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import eu.semiramis.policyManager.pdp.ConfigurationException;
import eu.semiramis.policyManager.pdp.ContextHandler;
import eu.semiramis.policyManager.util.ConfigurationProperties;
import eu.semiramis.policyManager.util.PMUtils;

/**
 * Factory for {@link ContextHandler} objects.
 * <p>
 * The implementation class of objects returned by the factory can be configured
 * using the property name {@link ContextHandler#CONTEXT_HANDLER_CLASS_PROPERTY}.
 * 
 */
public class ContextHandlerFactory {

	final static Logger logger = LoggerFactory
			.getLogger(ContextHandlerFactory.class);

	/**
	 * Name of the default configuration file for the ContextHandler
	 * */
	public static final String CONTEXT_HANDLER_CONFIGURATION_FILE = "contextHandler.properties";

	public static final Class DEFAULT_CONTEXT_HANDLER_CLASS = DefaultContextHandlerImpl.class;

	/**
	 * Creates a new {@link ContextHandler} basing on the default configuration
	 * file
	 * 
	 * @return the new {@link ContextHandler}
	 * @throws ClassNotFoundException
	 *             if the {@link ContextHandler} implementation class cannot be
	 *             found
	 * @throws IllegalAccessException
	 *             if the {@link ContextHandler} implementation class cannot be
	 *             instantiated
	 * @throws InstantiationException
	 *             if the {@link ContextHandler} implementation class cannot be
	 *             instantiated
	 * @throws ConfigurationException
	 *             if an exception occurs in configuring the
	 *             {@link ContextHandler} implementation
	 * @throws URISyntaxException
	 *             if the resource name is not well formed
	 * @throws IOException
	 *             if the load operation fails
	 */
	public static ContextHandler createContextHandler()
			throws ClassNotFoundException, IllegalAccessException,
			InstantiationException, ConfigurationException, IOException,
			URISyntaxException {
		return createContextHandler(CONTEXT_HANDLER_CONFIGURATION_FILE);
	}

	/**
	 * Creates a new {@link ContextHandler} basing on the custom configuration
	 * file provided
	 * 
	 * @param configurationFileName
	 *            the configuration file name
	 * @return the new {@link ContextHandler}
	 * @throws ClassNotFoundException
	 *             if the {@link ContextHandler} implementation class cannot be
	 *             found
	 * @throws IllegalAccessException
	 *             if the {@link ContextHandler} implementation class cannot be
	 *             instantiated
	 * @throws InstantiationException
	 *             if the {@link ContextHandler} implementation class cannot be
	 *             instantiated
	 * @throws ConfigurationException
	 *             if an exception occurs in configuring the
	 *             {@link ContextHandler} implementation
	 * @throws URISyntaxException
	 *             if the resource name is not well formed
	 * @throws IOException
	 *             if the load operation fails
	 * @throws Exception
	 *             if an exception occurs when loading the properties
	 */
	public static ContextHandler createContextHandler(
			String configurationFileName) throws ClassNotFoundException,
			IllegalAccessException, InstantiationException,
			ConfigurationException, IOException, URISyntaxException {
		return createContextHandler(new ConfigurationProperties(
				configurationFileName));
	}

	/**
	 * Creates a new {@link ContextHandler} basing on the custom configuration
	 * object provided
	 * 
	 * @param properties
	 *            the custom configuration properties object
	 * @return the new {@link ContextHandler}
	 * @throws ClassNotFoundException
	 *             if the {@link ContextHandler} implementation class cannot be
	 *             found
	 * @throws IllegalAccessException
	 *             if the {@link ContextHandler} implementation class cannot be
	 *             instantiated
	 * @throws InstantiationException
	 *             if the {@link ContextHandler} implementation class cannot be
	 *             instantiated
	 * @throws ConfigurationException
	 *             if an exception occurs in configuring the
	 *             {@link ContextHandler} implementation
	 */
	public static ContextHandler createContextHandler(Properties properties)
			throws ConfigurationException {

		logger.info("creating a new ContextHandler using properties \n{}",
				properties);

		String handlerClassName = properties
				.getProperty(ContextHandler.CONTEXT_HANDLER_CLASS_PROPERTY);

		// if no context handler found use the default one
		if (handlerClassName == null) {
			handlerClassName = DEFAULT_CONTEXT_HANDLER_CLASS.getName();
			logger.info("ContextHandler implementation class not found in configuration, using the default one: "
					+ DEFAULT_CONTEXT_HANDLER_CLASS.getName());
		}

		ContextHandler handler;
		try {
			handler = PMUtils.create(ContextHandler.class, handlerClassName);
		} catch (Exception e) {
			throw new ConfigurationException(
					"Cannot create the Context Handler", e);
		}

		handler.setConfigurationProperties(properties);

		return handler;
	}

}
