/**
 *    Copyright (C) 2012, 2013 Engineering Ingeneria Informatica s.p.a.
 *
 *    Authors:
 *     Paolo Roccetti <paolo.roccetti@eng.it>
 *     Silvio Sorace <silvio.sorace@eng.it>
 *
 *    This file is part of SEMIRAMIS Policy Management library (PM).
 *
 *    PM is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU Lesser General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    PM is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *    GNU Lesser General Public License for more details.
 *
 *    You should have received a copy of the GNU Lesser General Public License
 *    along with PM. If not, see <http://www.gnu.org/licenses/>.
 * 
 */
package eu.semiramis.policyManager.pdp.jboss;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;

import org.jboss.security.xacml.factories.RequestResponseContextFactory;
import org.jboss.security.xacml.interfaces.PolicyDecisionPoint;
import org.jboss.security.xacml.interfaces.RequestContext;
import org.jboss.security.xacml.interfaces.ResponseContext;
import org.opensaml.xacml.ctx.RequestType;
import org.opensaml.xacml.ctx.ResponseType;
import org.opensaml.xml.io.UnmarshallingException;
import org.opensaml.xml.parse.XMLParserException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import eu.semiramis.policyManager.pdp.EvaluationException;
import eu.semiramis.policyManager.pdp.PDP;
import eu.semiramis.policyManager.pdp.impl.AbstractPDP;
import eu.semiramis.policyManager.util.opensaml.XACMLResponseUtils;

/**
 * This concrete implementation of the PDP uses a JBoss XACML engine to evaluate
 * access control requests against the current XACML policies
 * 
 */
public class JBossPDP extends AbstractPDP {

	final Logger logger = LoggerFactory.getLogger(JBossPDP.class);

	/**
	 * The inner JBoss XACML PDP
	 * */
	private PolicyDecisionPoint pdp;

	/**
	 * Creates a new instance of a {@link JBossPDP}
	 */
	JBossPDP(String pdpName, PolicyDecisionPoint internalPDP) {
		super(pdpName);
		this.pdp = internalPDP;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see eu.semiramis.policyManager.pdp.PDP#evaluate(org.opensaml.xacml.ctx.
	 * RequestType)
	 */
	@Override
	public ResponseType evaluate(RequestType request)
			throws EvaluationException {

		logger.info("translating openSAML request {} to JBoss XACML", request);

		// translate request to the JBoss XACML object
		RequestContext reqCtx;
		try {
			reqCtx = translate(request);
		} catch (IOException e) {
			throw new EvaluationException(
					"Cannot translate openSAML request to the JBoss format: "
							+ e.getMessage(), e);
		}

		logger.info(
				"openSAML request {} translated to JBoss XACML request {}:\n{}",
				new Object[] { request, reqCtx, marshall(reqCtx) });

		// evaluate request
		ResponseContext respCtx = pdp.evaluate(reqCtx);

		logger.info("evaluation of request {} using JBossPDP returned {}:\n{}",
				new Object[] { reqCtx, respCtx, marshall(respCtx) });

		logger.info("translating JBossXACML response {} to openSAML", respCtx);

		// translate response to the openSAML format
		ResponseType response;
		try {
			response = translate(respCtx);
		} catch (Exception e) {
			throw new EvaluationException(
					"Cannot translate JBoss response to openSAML format: "
							+ e.getMessage(), e);
		}

		logger.info(
				"JBossXACML response {} translated to openSAML response {}",
				respCtx, response);

		return response;

	}

	/**
	 * Translate the openSAML XACML request to the JBoss XACML
	 */
	private RequestContext translate(RequestType openSAMLrequest)
			throws IOException {

		// create the jboss request
		RequestContext jbossRequest = RequestResponseContextFactory
				.createRequestCtx();

		jbossRequest.readRequest(openSAMLrequest.getDOM());

		return jbossRequest;
	}

	/**
	 * Translate the JBossXACML response to the openSAML XACML
	 * 
	 * @throws IOException
	 * @throws XMLParserException
	 * @throws UnmarshallingException
	 */
	private ResponseType translate(ResponseContext jbossResponse)
			throws IOException, XMLParserException, UnmarshallingException {

		// get the response Element
		ByteArrayOutputStream stream = new ByteArrayOutputStream();
		jbossResponse.marshall(stream);

		// create the openSAML response
		ResponseType openSAMLResponse = XACMLResponseUtils
				.deserialize(new ByteArrayInputStream(stream.toByteArray()));

		// return the response
		return openSAMLResponse;

	}

	// marshall a request to a String
	private String marshall(RequestContext reqCtx) {
		ByteArrayOutputStream stream = new ByteArrayOutputStream();
		try {
			reqCtx.marshall(stream);
		} catch (IOException e) {
			logger.error("Exception mashalling request {}", reqCtx, e);
		}
		return stream.toString();
	}

	// marshall a response to a String
	private String marshall(ResponseContext respCtx) {
		ByteArrayOutputStream stream = new ByteArrayOutputStream();
		try {
			respCtx.marshall(stream);
		} catch (IOException e) {
			logger.error("Exception mashalling response {}", respCtx, e);
		}
		return stream.toString();
	}

	/* (non-Javadoc)
	 * @see eu.semiramis.policyManager.pdp.PDP#destroy()
	 */
	@Override
	public void destroy() {
		//currently do nothing
	}
}
