/**
 *    Copyright (C) 2012, 2013 Engineering Ingeneria Informatica s.p.a.
 *
 *    Authors:
 *     Paolo Roccetti <paolo.roccetti@eng.it>
 *     Silvio Sorace <silvio.sorace@eng.it>
 *
 *    This file is part of SEMIRAMIS Policy Management library (PM).
 *
 *    PM is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU Lesser General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    PM is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *    GNU Lesser General Public License for more details.
 *
 *    You should have received a copy of the GNU Lesser General Public License
 *    along with PM. If not, see <http://www.gnu.org/licenses/>.
 * 
 */
package eu.semiramis.policyManager.pdp.jboss;

import java.util.HashSet;
import java.util.Properties;
import java.util.Set;

import org.jboss.security.xacml.interfaces.PolicyDecisionPoint;
import org.jboss.security.xacml.interfaces.PolicyLocator;
import org.opensaml.xacml.ctx.RequestType;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import eu.semiramis.policyManager.pdp.ConfigurationException;
import eu.semiramis.policyManager.pdp.PDP;
import eu.semiramis.policyManager.pdp.PDPFactory;
import eu.semiramis.policyManager.util.PMUtils;

/**
 * This concrete implementation of the {@link PDPFactory} creates a {@link PDP}
 * that uses a JBoss XACML engine to evaluate access control requests against
 * the current XACML policies
 * 
 */
public class JBossPDPFactory implements PDPFactory {

	final Logger logger = LoggerFactory.getLogger(JBossPDPFactory.class);

	/* (non-Javadoc)
	 * @see eu.semiramis.policyManager.pdp.PDPFactory#getPDP(java.lang.String, eu.semiramis.policyManager.pdp.ContextHandler, java.util.Properties, org.opensaml.xacml.ctx.RequestType)
	 */
	@Override
	public PDP getPDP(String pdpName,
			eu.semiramis.policyManager.pdp.ContextHandler handler,
			Properties configurationProperties, RequestType request)
			throws ConfigurationException {

		logger.info("creating new JBossPDP for request {} using properties {}",
				request, configurationProperties);

		// Create Locators
		Set<PolicyLocator> locators = new HashSet<PolicyLocator>();

		// create locator factory
		LocatorFactory factory;
		try {
			factory = PMUtils
					.create(LocatorFactory.class,
							configurationProperties
									.getProperty(pdpName + LocatorFactory.JBOSS_LOCATOR_FACTORY_CLASS_PROPERTY));
		} catch (Exception e) {
			throw new ConfigurationException(
					"Cannot create the LocatorFactory to get Locator", e);
		}

		logger.info(
				"using LocatorFactory {} to get PolicyLocator for request {}",
				factory, request);

		// create locator
		PolicyLocator locator = factory.getLocator(configurationProperties);

		logger.info("using PolicyLocator {} to locate policies for request {}",
				locator, request);

		// add locator to locators
		locators.add(locator);

		// create internalPDP
		PolicyDecisionPoint internalPDP = new org.jboss.security.xacml.core.JBossPDP();

		// add locator to PDP
		internalPDP.setLocators(locators);

		return new JBossPDP(pdpName, internalPDP);

	}

}
