/**
 *    Copyright (C) 2012, 2013 Engineering Ingeneria Informatica s.p.a.
 *
 *    Authors:
 *     Paolo Roccetti <paolo.roccetti@eng.it>
 *     Silvio Sorace <silvio.sorace@eng.it>
 *
 *    This file is part of SEMIRAMIS Policy Management library (PM).
 *
 *    PM is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU Lesser General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    PM is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *    GNU Lesser General Public License for more details.
 *
 *    You should have received a copy of the GNU Lesser General Public License
 *    along with PM. If not, see <http://www.gnu.org/licenses/>.
 * 
 */
package eu.semiramis.policyManager.util;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.Date;
import java.util.Enumeration;
import java.util.Properties;
import java.util.Set;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Extension of {@link Properties} that load properties from a resource (e.g. a
 * file). Each time a property is asked, the resource is checked for
 * modifications and properties are reloaded if needed.
 * 
 */
public class ConfigurationProperties extends Properties {

	final Logger logger = LoggerFactory
			.getLogger(ConfigurationProperties.class);

	private String resourceName;

	private Properties properties;

	private long lastLoadTime;

	/**
	 * Creates a new {@link ConfigurationProperties} linked to the given
	 * resource
	 * 
	 * @param resourceName
	 *            the resource name
	 * @throws URISyntaxException
	 *             if the resource name is not well formed
	 * @throws IOException
	 *             if the load operation fails
	 */
	public ConfigurationProperties(String resourceName) throws IOException,
			URISyntaxException {
		super();

		this.resourceName = resourceName;

		logger.info("creating new ConfigurationProperties from resource {}",
				resourceName);

		load();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.util.Properties#getProperty(java.lang.String)
	 */
	@Override
	public String getProperty(String key) {

		loadIfChanged();

		return this.properties.getProperty(key);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.util.Properties#getProperty(java.lang.String, java.lang.String)
	 */
	@Override
	public String getProperty(String key, String defaultValue) {

		loadIfChanged();

		return this.properties.getProperty(key, defaultValue);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.util.Properties#propertyNames()
	 */
	@Override
	public Enumeration<?> propertyNames() {

		loadIfChanged();

		return this.properties.propertyNames();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.util.Properties#stringPropertyNames()
	 */
	@Override
	public Set<String> stringPropertyNames() {

		loadIfChanged();

		return this.properties.stringPropertyNames();
	}
	
	

	/* (non-Javadoc)
	 * @see java.util.Hashtable#equals(java.lang.Object)
	 */
	@Override
	public synchronized boolean equals(Object o) {
		
		loadIfChanged();
		
		return super.equals(o);
	}

	// reload properties if needed
	private void loadIfChanged() {

		URL fileURL = null;
		try {
			fileURL = Thread.currentThread().getContextClassLoader()
					.getResource(this.resourceName);

//			logger.info("properties for resource {} are located to URL {}",
//					this.resourceName, fileURL);

			File file = new File(fileURL.toURI());

			if (file.lastModified() > this.lastLoadTime) {
				load();
			}
		} catch (Exception e) {
			throw new RuntimeException("Cannot load properties from resource "
					+ this.resourceName + "(URL:" + fileURL + ")", e);
		}

	}

	// reload properties (synchronized to avoid conflicts)
	private synchronized void load() throws IOException, URISyntaxException {

		Properties newProperties = new Properties();

		URL fileURL = Thread.currentThread().getContextClassLoader()
				.getResource(this.resourceName);

		logger.info("loading properties from resource {} (URL: {})",
				this.resourceName, fileURL);

		File file = new File(fileURL.toURI());

		newProperties.load(new FileInputStream(file));

		this.properties = newProperties;
		this.lastLoadTime = file.lastModified();

		logger.info("properties loaded {}", this);

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.util.Hashtable#toString()
	 */
	@Override
	public synchronized String toString() {
		return "[resource: " + this.resourceName + ", loadTime: "
				+ (new Date(this.lastLoadTime)).toString() + ", properties:\n"
				+ this.properties.toString();
	}

	
}
