/**
 *    Copyright (C) 2012, 2013 Engineering Ingeneria Informatica s.p.a.
 *
 *    Authors:
 *     Paolo Roccetti <paolo.roccetti@eng.it>
 *     Silvio Sorace <silvio.sorace@eng.it>
 *
 *    This file is part of SEMIRAMIS Policy Management library (PM).
 *
 *    PM is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU Lesser General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    PM is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *    GNU Lesser General Public License for more details.
 *
 *    You should have received a copy of the GNU Lesser General Public License
 *    along with PM. If not, see <http://www.gnu.org/licenses/>.
 * 
 */
package eu.semiramis.policyManager.util;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import eu.semiramis.policyManager.pdp.ContextHandler;

/**
 * This class contains some utility method for policy manager classes.
 * 
 */
public class PMUtils {

	final static Logger logger = LoggerFactory.getLogger(PMUtils.class);

	/**
	 * Creates a new instance of the class from the
	 * <code>implementationClassName</code>, and return it as implementation of
	 * the interface <code>interfaceClass</code>.
	 * <p>
	 * 
	 * @param implementationClassName
	 *            the name of the implementation class
	 * @param interfaceClass the interface to return, the implementationClassName is
	 *        checked to ensure it implements the given interface
	 * @throws IllegalAccessException
	 *             if the {@link ContextHandler} implementation class cannot be
	 *             instantiated
	 * @throws InstantiationException
	 *             if the {@link ContextHandler} implementation class cannot be
	 *             instantiated
	 * @throws ClassNotFoundException
	 *             if the {@link ContextHandler} implementation class cannot be
	 *             found
	 * 
	 * */
	@SuppressWarnings("unchecked")
	public static <T> T create(Class<T> interfaceClass,
			String implementationClassName) throws ClassNotFoundException,
			InstantiationException, IllegalAccessException {

		if (implementationClassName == null)
			throw new NullPointerException(
					"The implementation class name cannot be null");

		T newObject;

		logger.info("creating new object of class {} instanceof {}",
				implementationClassName, interfaceClass.getName());

		@SuppressWarnings("rawtypes")
		Class implementationClass = Class.forName(implementationClassName);

		// check the configured class compatibility with the target class, and
		// create the new instance
		if (interfaceClass.isAssignableFrom(implementationClass)) {
			newObject = (T) implementationClass.newInstance();
		} else {
			throw new IllegalArgumentException("The class "
					+ implementationClassName
					+ " does not implements the interface "
					+ interfaceClass.getName());
		}

		return newObject;

	}
}
