/**
 *    Copyright (C) 2012, 2013 Engineering Ingeneria Informatica s.p.a.
 *
 *    Authors:
 *     Paolo Roccetti <paolo.roccetti@eng.it>
 *     Silvio Sorace <silvio.sorace@eng.it>
 *
 *    This file is part of SEMIRAMIS Policy Management library (PM).
 *
 *    PM is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU Lesser General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    PM is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *    GNU Lesser General Public License for more details.
 *
 *    You should have received a copy of the GNU Lesser General Public License
 *    along with PM. If not, see <http://www.gnu.org/licenses/>.
 * 
 */
package eu.semiramis.policyManager.util.opensaml;

import java.io.InputStream;
import java.util.List;

import javax.xml.namespace.QName;

import org.opensaml.Configuration;
import org.opensaml.xacml.ctx.ActionType;
import org.opensaml.xacml.ctx.AttributeType;
import org.opensaml.xacml.ctx.AttributeValueType;
import org.opensaml.xacml.ctx.RequestType;
import org.opensaml.xacml.ctx.ResourceType;
import org.opensaml.xacml.ctx.SubjectType;
import org.opensaml.xml.XMLObject;
import org.opensaml.xml.io.Unmarshaller;
import org.opensaml.xml.io.UnmarshallingException;
import org.opensaml.xml.parse.BasicParserPool;
import org.opensaml.xml.parse.XMLParserException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * Utility methods to deal with openSAML XACML Requests
 * 
 */
public class XACMLRequestUtils {

	/**
	 * Deserialize a XACML request from a stream
	 * 
	 * @param requestStream
	 *            the stream containing the XACML request
	 * @throws XMLParserException
	 *             if an exception occurs during parsing
	 * @throws UnmarshallingException
	 *             if an exception occus during unmarshalling
	 * */
	public static RequestType deserialize(InputStream requestStream)
			throws XMLParserException, UnmarshallingException {

		// Get parser pool manager
		BasicParserPool parser = new BasicParserPool();
		parser.setNamespaceAware(true);

		// Parse byte array
		Document requestDocument = parser.parse(requestStream);
		Element requestElement = requestDocument.getDocumentElement();

		// get the openSAML unmarshaller for the request element
		Unmarshaller unmarshaller = Configuration.getUnmarshallerFactory()
				.getUnmarshaller(RequestType.DEFAULT_ELEMENT_NAME);

		// unmarshall the request
		RequestType openSAMLRequest = (RequestType) unmarshaller
				.unmarshall(requestElement);

		return openSAMLRequest;
	}

	/**
	 * Sets the attribute (id and value) in the access category indicated. If
	 * the access category is not there, it will be created.
	 * 
	 * If an attribute with the same ID is already there it will be replaced.
	 * 
	 * @param request
	 *            the request
	 * @param attributeId
	 *            the attribute id
	 * @param attributeValue
	 *            the attribute value
	 * @param subjectCategory
	 *            the subject category
	 */
	public static void setAttributeInSubject(RequestType request,
			String attributeId, String attributeValue, String subjectCategory) {

		// find the access subject element
		SubjectType accessSubject = getAccessSubject(request, subjectCategory);

		// create the ideas subject if not there, and add it to the request
		if (accessSubject == null) {
			accessSubject = createAndSet(request.getSubjects(),
					SubjectType.class, SubjectType.DEFAULT_ELEMENT_NAME);
			accessSubject.setSubjectCategory(subjectCategory);
		}

		// set attribute in subject
		setAttributeInList(accessSubject.getAttributes(), attributeId,
				attributeValue);

	}

	/**
	 * Get the first attribute value for the attribute with given id in the
	 * access category indicated.
	 * 
	 * @param request
	 *            the request
	 * @param attributeId
	 *            the attribute id
	 * @param accessCategory
	 *            the access category
	 * @return the attribute from subject
	 */
	public static String getAttributeFromSubject(RequestType request,
			String attributeId, String accessCategory) {

		// find the access subject element
		SubjectType accessSubject = getAccessSubject(request, accessCategory);

		// if the access subject is not set return null
		if (accessSubject == null)
			return null;

		// get the attribute, if any
		AttributeType attribute = getAttributeFromList(
				accessSubject.getAttributes(), attributeId);

		return getFirstValue(attribute);

	}

	/**
	 * set the attribute (id and value) in action. If an attribute with the same
	 * ID is already there it will be replaced.
	 * 
	 * @param request
	 *            the request
	 * @param attributeId
	 *            the attribute id
	 * @param attributeValue
	 *            the attribute value
	 */
	public static void setAttributeInAction(RequestType request,
			String attributeId, String attributeValue) {
		ActionType action = request.getAction();

		// create the action element, if null
		if (action == null) {
			action = XACMLUtils.createAction();
			request.setAction(action);
		}

		// set attribute in action
		setAttributeInList(action.getAttributes(), attributeId, attributeValue);

	}

	/**
	 * Get the first value of the attribute id from action attributes
	 * 
	 * @param request
	 *            the request
	 * @param attributeId
	 *            the attribute id
	 * @return the from action
	 */
	public static String getAttributeFromAction(RequestType request,
			String attributeId) {

		ActionType action = request.getAction();

		if (action == null)
			return null;

		// get the attribute, if any
		AttributeType attribute = getAttributeFromList(action.getAttributes(),
				attributeId);

		return getFirstValue(attribute);
	}

	/**
	 * set the attribute (id and value) in resource attributes. If an attribute
	 * with the same ID is already there it will be replaced.
	 * 
	 * @param request
	 *            the request
	 * @param attributeId
	 *            the attribute id
	 * @param attributeValue
	 *            the attribute value
	 */
	public static void setAttributeInResource(RequestType request,
			String attributeId, String attributeValue) {

		// find the resource element, if any
		ResourceType resource = getResource(request);

		// create the resource if not there, and add it to the request
		if (resource == null) {
			resource = createAndSet(request.getResources(), ResourceType.class,
					ResourceType.DEFAULT_ELEMENT_NAME);
		}

		// set attribute in resource
		setAttributeInList(resource.getAttributes(), attributeId,
				attributeValue);
	}

	/**
	 * Get the first value of the attribute id from resource attributes
	 * 
	 * @param request
	 *            the request
	 * @param attributeId
	 *            the attribute id
	 * @return the from resource
	 */
	public static String getAttributeFromResource(RequestType request,
			String attributeId) {

		// find the resource element, if any
		ResourceType resource = getResource(request);

		// if the resource is not set return null
		if (resource == null)
			return null;

		// get the attribute, if any
		AttributeType attribute = getAttributeFromList(
				resource.getAttributes(), attributeId);

		return getFirstValue(attribute);

	}

	// set the given attribute (id and value) in the list
	public static void setAttributeInList(List<AttributeType> attributes,
			String attributeID, String attributeValue) {

		// check attribute value
		if (attributeValue == null)
			throw new NullPointerException("The attribute value cannot be null");

		// check attribute value
		if (attributeID == null)
			throw new NullPointerException("The attribute id cannot be null");

		// check if the attribute is already there
		AttributeType attribute = getAttributeFromList(attributes, attributeID);

		if (attribute == null) {
			// create the attribute object
			AttributeType newAttribute = XACMLUtils.createAttribute(
					attributeID,
					XACMLUtils.createAttributeValue(attributeValue));

			// add the new attribute
			attributes.add(newAttribute);

		} else {
			// clear current values
			attribute.getAttributeValues().clear();
			// add the new value
			attribute.getAttributeValues().add(
					XACMLUtils.createAttributeValue(attributeValue));
		}

	}

	/**
	 * Return the {@link AttributeType}, if any, with the given attribute id,
	 * null otherwise
	 * 
	 * @param attributes
	 *            the attributes
	 * @param attributeId
	 *            the attribute id
	 * @return the attribute, or null if the attribute cannot be found
	 */
	public static AttributeType getAttributeFromList(
			List<AttributeType> attributes, String attributeId) {

		// iterate over attribute list
		for (AttributeType attribute : attributes) {
			// if the attribute matches the given name
			if (attributeId.equals(attribute.getAttributeID())) {
				return attribute;
			}
		}
		// if the attribute is not found return null
		return null;
	}

	// Get the access subject, if any
	private static SubjectType getAccessSubject(RequestType request,
			String accessCategory) {

		// get current subjects
		List<SubjectType> subjects = request.getSubjects();

		// iterate over subjects
		for (SubjectType subject : subjects) {
			// if found, return the access subject
			if (accessCategory.equals(subject.getSubjectCategory())) {
				return subject;
			}
		}

		return null;
	}

	// Gets the resource, if any.
	private static ResourceType getResource(RequestType request) {

		// get the list of resources
		List<ResourceType> resources = request.getResources();

		// if the list is empty return null
		if (resources.isEmpty())
			return null;

		// return the first element in the list
		return resources.get(0);

	}

	private static ResourceType getFirstResource(RequestType request) {

		// get the list of resources
		List<ResourceType> resources = request.getResources();

		// if the list is empty return null
		if (resources.isEmpty())
			return null;

		// return the first element in the list
		return resources.get(0);

	}

	// Get the first attribute value, null if not found or attribute is null
	private static String getFirstValue(AttributeType attribute) {

		if (attribute == null)
			return null;

		List<AttributeValueType> values = attribute.getAttributeValues();

		if (values.size() == 0)
			return null;

		return values.get(0).getValue();
	}

	/**
	 * create the element of the given class and add it in the list
	 * 
	 * @param list
	 *            the list where to add the element to
	 * @param cls
	 *            the {@link Class} of the element to create
	 * @param name
	 *            the {@link QName} of the element to create
	 * @return the element created
	 */
	private static <T extends XMLObject> T createAndSet(List<T> list,
			Class<T> cls, QName name) {

		// create the element
		T t = XACMLUtils.create(cls, name);

		// add the element to the list
		list.add(t);

		return t;
	}

}
