/**
 *    Copyright (C) 2012, 2013 Engineering Ingeneria Informatica s.p.a.
 *
 *    Authors:
 *     Paolo Roccetti <paolo.roccetti@eng.it>
 *     Silvio Sorace <silvio.sorace@eng.it>
 *
 *    This file is part of SEMIRAMIS Policy Management library (PM).
 *
 *    PM is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU Lesser General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    PM is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *    GNU Lesser General Public License for more details.
 *
 *    You should have received a copy of the GNU Lesser General Public License
 *    along with PM. If not, see <http://www.gnu.org/licenses/>.
 * 
 */
package eu.semiramis.policyManager.util.opensaml;

import java.io.StringReader;
import java.util.Arrays;

import javax.xml.namespace.QName;

import org.opensaml.Configuration;
import org.opensaml.xacml.ctx.ActionType;
import org.opensaml.xacml.ctx.AttributeType;
import org.opensaml.xacml.ctx.AttributeValueType;
import org.opensaml.xacml.ctx.ResourceType;
import org.opensaml.xacml.ctx.SubjectType;
import org.opensaml.xml.XMLObject;
import org.opensaml.xml.XMLObjectBuilder;
import org.opensaml.xml.io.Marshaller;
import org.opensaml.xml.io.MarshallerFactory;
import org.opensaml.xml.io.MarshallingException;
import org.opensaml.xml.io.Unmarshaller;
import org.opensaml.xml.io.UnmarshallerFactory;
import org.opensaml.xml.io.UnmarshallingException;
import org.opensaml.xml.parse.BasicParserPool;
import org.opensaml.xml.parse.XMLParserException;
import org.opensaml.xml.util.XMLHelper;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * The {@link XACMLUtils} class contains some utility method that helps in
 * building XACML objects.
 */
public class XACMLUtils {

	/**
	 * Creates a {@link SubjectType} object containing the given attributes
	 * 
	 * @param attributes
	 *            the attributes to be added to the subject object
	 * @return the {@link SubjectType} object created
	 */
	public static SubjectType createSubject(AttributeType... attributes) {

		// build the subject element
		SubjectType subject = create(SubjectType.class,
				SubjectType.DEFAULT_ELEMENT_NAME);

		// set subject attributes
		subject.getAttributes().addAll(Arrays.asList(attributes));

		return subject;

	}

	/**
	 * Creates a {@link ResourceType} object containing the given attributes
	 * 
	 * @param attributes
	 *            the attributes to be added to the resource object
	 * @return the {@link ResourceType} object created
	 */
	public static ResourceType createResource(AttributeType... attributes) {

		// build the resource element
		ResourceType resource = create(ResourceType.class,
				ResourceType.DEFAULT_ELEMENT_NAME);

		// set resource attributes
		resource.getAttributes().addAll(Arrays.asList(attributes));

		return resource;

	}

	/**
	 * Creates an {@link AttributeType} object containing the given values
	 * 
	 * @param attributeId
	 *            the attribute id
	 * @param values
	 *            the attribute values
	 * @return the {@link AttributeType} object created
	 */
	public static AttributeType createAttribute(String attributeId,
			AttributeValueType... values) {

		AttributeType attribute = create(AttributeType.class,
				AttributeType.DEFAULT_ELEMENT_NAME);

		// set attribute id
		attribute.setAttributeID(attributeId);

		// set attribute values
		attribute.getAttributeValues().addAll(Arrays.asList(values));

		return attribute;
	}

	/**
	 * Creates an {@link AttributeValueType} object with the given value
	 * 
	 * @param attributeValue
	 *            the attribute value
	 * @return the {@link AttributeValueType} object created
	 */
	public static AttributeValueType createAttributeValue(String attributeValue) {

		AttributeValueType av = create(AttributeValueType.class,
				AttributeValueType.DEFAULT_ELEMENT_NAME);

		// set value
		av.setValue(attributeValue);

		return av;
	}

	/**
	 * Creates an {@link ActionType} object containing the given attributes
	 * 
	 * @param attributes
	 *            the attributes to be added to the {@link ActionType} object
	 * @return the {@link ActionType} object created
	 */
	public static ActionType createAction(AttributeType... attributes) {

		// build the Action element
		ActionType action = create(ActionType.class,
				ActionType.DEFAULT_ELEMENT_NAME);

		// set action attributes
		action.getAttributes().addAll(Arrays.asList(attributes));

		return action;

	}

	/**
	 * Creates a new {@link XMLObject} from its {@link Class} object and
	 * {@link QName}.
	 * 
	 * @param cls
	 *            the {@link Class} of the new {@link XMLObject}
	 * @param qname
	 *            the {@link QName} of the new {@link XMLObject}
	 * @return the new {@link XMLObject} created
	 */
	@SuppressWarnings("unchecked")
	public static <T extends XMLObject> T create(Class<T> cls, QName qname) {
		return (T) ((XMLObjectBuilder) Configuration.getBuilderFactory()
				.getBuilder(qname)).buildObject(qname);
	}

	/**
	 * Marshal an XMLObject to a W3C DOM {@link Element}.
	 * 
	 * @param xmlObject
	 *            the object extending {@link XMLObject} to marshall
	 * @return the W3C DOM element representing this XML element
	 * @throws MarshallingException
	 *             if the marshalling process fails
	 */
	public static <T extends XMLObject> Element marshall(T xmlObject)
			throws MarshallingException {

		// Get the marshaller factory
		MarshallerFactory marshallerFactory = Configuration
				.getMarshallerFactory();

		// Get the Request marshaller
		Marshaller marshaller = marshallerFactory.getMarshaller(xmlObject);

		// Marshall the request
		return marshaller.marshall(xmlObject);

	}

	/**
	 * Marshall and print an extension of {@link XMLObject} to a String.
	 * 
	 * @param xmlObject
	 *            the xml object extending {@link XMLObject}
	 * @return the string of the marshalled object
	 * @throws MarshallingException
	 *             if the marshalling process fails
	 */
	public static <T extends XMLObject> String printXMLObject(T xmlObject)
			throws MarshallingException {
		return XMLHelper.prettyPrintXML(marshall(xmlObject));
	}

	/**
	 * Unmarshall an extension of a {@link XMLObject} from its {@link Document}
	 * representation
	 * 
	 * @param document
	 *            the {@link Document} to unmarshall
	 * 
	 * @return the {@link XMLObject} from the unmashalled {@link Document}
	 * 
	 * @throws UnmarshallingException
	 *             if the unmarshalling process fails
	 * */
	public static <T extends XMLObject> T unmarshall(Document document)
			throws UnmarshallingException {

		// gt the document root element
		Element rootElement = document.getDocumentElement();

		// Get the unmarshaller factory
		UnmarshallerFactory unmarshallerFactory = Configuration
				.getUnmarshallerFactory();

		// get the unmarshaller
		Unmarshaller unmarshaller = unmarshallerFactory
				.getUnmarshaller(rootElement);

		// Unmarshall using the document root element
		@SuppressWarnings("unchecked")
		T xmlObject = (T) unmarshaller.unmarshall(rootElement);

		return xmlObject;
	}

	/**
	 * Unmarshall an extension of a {@link XMLObject} from its {@link String}
	 * representation
	 * 
	 * @param strXMLDocument
	 *            the {@link String} containing the XML representation of the
	 *            {@link XMLObject} to unmarshall
	 * 
	 * @return the {@link XMLObject} from the given {@link String}
	 * 
	 * @throws XMLParserException
	 *             in case the XML string cannot be properly parsed to a
	 *             {@link Document}
	 * @throws UnmarshallingException
	 *             if the unmarshalling process fails
	 * 
	 * 
	 * */
	public static <T extends XMLObject> T unmarshall(String strXMLDocument)
			throws XMLParserException, UnmarshallingException {

		// get the parser
		BasicParserPool parser = new BasicParserPool();
		parser.setNamespaceAware(true);

		// parse the string
		Document document = parser.parse(new StringReader(strXMLDocument));

		return unmarshall(document);
	}

}
